
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>
#include <gst/mse/mse-prelude.h>

G_BEGIN_DECLS

/* enumerations from "gstmediasource.h" */

GST_MSE_API
GType gst_media_source_ready_state_get_type (void);
#define GST_TYPE_MEDIA_SOURCE_READY_STATE (gst_media_source_ready_state_get_type())

GST_MSE_API
GType gst_media_source_error_get_type (void);
#define GST_TYPE_MEDIA_SOURCE_ERROR (gst_media_source_error_get_type())

GST_MSE_API
GType gst_media_source_eos_error_get_type (void);
#define GST_TYPE_MEDIA_SOURCE_EOS_ERROR (gst_media_source_eos_error_get_type())

/* enumerations from "gstmsesrc.h" */

GST_MSE_API
GType gst_mse_src_ready_state_get_type (void);
#define GST_TYPE_MSE_SRC_READY_STATE (gst_mse_src_ready_state_get_type())

/* enumerations from "gstsourcebuffer.h" */

GST_MSE_API
GType gst_source_buffer_append_mode_get_type (void);
#define GST_TYPE_SOURCE_BUFFER_APPEND_MODE (gst_source_buffer_append_mode_get_type())

G_END_DECLS

/* Generated data ends here */

